from types import SimpleNamespace

ConfigEnvironmentVariables = SimpleNamespace(
    console_url="GALILEO_CONSOLE_URL",
    username="GALILEO_USERNAME",
    password="GALILEO_PASSWORD",
    api_key="GALILEO_API_KEY",
)
